from pprint import pprint
from typing import Any, Dict, List


def audit_data(data: Dict[str, Any], ignore: List[str] = []) -> None:
    """Print a row if any of the fields not ignored are still unused."""
    cleaned = {}
    for key, value in data.items():
        if key in ignore:
            continue
        if value is None or value == "":
            continue
        cleaned[key] = value
    if len(cleaned):
        pprint(cleaned)



import orjson
from typing import BinaryIO
from followthemoney.proxy import E


def write_entity(fh: BinaryIO, entity: E) -> None:
    data = entity.to_dict()
    entity_id = data.pop("id")
    assert entity_id is not None, data
    sort_data = dict(id=entity_id)
    sort_data.update(data)
    out = orjson.dumps(sort_data, option=orjson.OPT_APPEND_NEWLINE)
    fh.write(out)



import logging
from typing import Generator, Optional
from contextlib import contextmanager
from followthemoney.proxy import EntityProxy

from zavod import settings
from zavod.context import Zavod
from zavod.logs import configure_logging, get_logger
from zavod.util import PathLike

__version__ = "0.2.1"
__all__ = [
    "init",
    "context",
    "Zavod",
    "PathLike",
    "configure_logging",
    "get_logger",
    "settings",
]


def init(
    name: str,
    prefix: Optional[str] = None,
    verbose: bool = False,
    data_path: PathLike = settings.DATA_PATH,
) -> Zavod[EntityProxy]:
    """Initiate the zavod working environment and create a processing context."""
    level = logging.DEBUG if verbose else logging.INFO
    configure_logging(level=level)
    return Zavod(name, EntityProxy, prefix=prefix, data_path=data_path)


@contextmanager
def init_context(
    name: str,
    prefix: Optional[str] = None,
    verbose: bool = False,
    data_path: PathLike = settings.DATA_PATH,
) -> Generator[Zavod[EntityProxy], None, None]:
    ctx = init(name, prefix=prefix, verbose=verbose, data_path=data_path)
    try:
        yield ctx
    finally:
        ctx.close()



import json
import click
import logging
import asyncio
from pathlib import Path
from typing import Optional, Generator, Tuple
from followthemoney import model
from followthemoney.proxy import EntityProxy

from zavod.logs import configure_logging, get_logger
from zavod.store import write_entity

log = get_logger(__name__)


@click.group(help="Zavod data factory")
def cli() -> None:
    configure_logging(level=logging.INFO)


@cli.command("sorted-merge", help="Merge sorted entities")
@click.argument("path", type=click.Path(exists=True, dir_okay=False, path_type=Path))
@click.option(
    "-o", "--out-path", type=click.Path(dir_okay=False, path_type=Path), default="-"
)
def apply(path: Path, out_path: Path) -> None:
    entity: Optional[EntityProxy] = None
    with open(out_path, "wb") as outfh:
        with open(path, "r") as fh:
            while True:
                line = fh.readline()
                if not line:
                    break
                data = json.loads(line)
                next_entity = EntityProxy.from_dict(model, data)
                if entity is None:
                    entity = next_entity
                    continue
                if next_entity.id == entity.id:
                    entity = entity.merge(next_entity)
                    continue
                write_entity(outfh, entity)
                entity = next_entity

        if entity is not None:
            write_entity(outfh, entity)
